'*******************************************************************************
'
' Module:       SLT DF9XS Controller 11.bas
'
' Hardware:     Stepper Loop Tuner 1.1
'               2 Schrittmotoren bipolar
'               2 Lichtschranken
'               2 x 2 AI fr SWR-Meter und Phasendetektor
'               2 Relais
'
' Date:         14.01.2012
'
' Description:  Anpaautomatik fr Magnetantenne
'
'*******************************************************************************
'
'Hardware-Deklaration
'--------------------
$regfile = "m48def.dat"                 'Typ Prozessor
$crystal = 1000000                      '1 MHz durch Fusebits
$baud = 300

'Deklarationen
'-------------
Dim A1(6) As Byte                       'Bitmuster fr Ansteuerung
Dim A2(6) As Byte                       'Schrittmotoren
Dim A3(6) As Byte

Dim P1 As Integer                       'Phasenzhler Schrittmotor 1
Dim P2 As Integer
Dim W1 As Integer                       'Sollwert Position
Dim W2 As Integer
Dim X1 As Integer                       'Istwert Position
Dim X2 As Integer
Dim T1 As Integer                       'Wartezeit nach Positionierung
Dim T2 As Integer
Dim B As String * 10                    'Zwischenvariable Sollwert
Dim M1 As Integer                       'Zwischenvariable Strangansteuerung
Dim M2 As Integer
Dim C As Integer                        'Vorlaufspannung SWR-Meter
Dim D As Integer                        'Rcklaufspannung SWR-Meter
Dim E As Integer                        'Vorlaufspannung Phasendetektor
Dim F As Integer                        'Rcklaufspannung Phasendetektor
Dim H As Integer                        'Zwischenvariable fr Relais
Dim Temp1 As Integer
Dim Temp2 As Integer
Dim Temp3 As Integer
Dim Temp4 As Integer
Dim T1min As Integer
Dim T2min As Integer
Dim Tmax As Integer
Dim Delta As Integer
Dim Tg As Integer
Dim I As Integer

Config Adc = Single , Prescaler = Auto , Reference = Avcc

Const Ein = 1
Const Aus = 0

Led1 Alias Portb.6                      'direkte Verwendung der Variablen
Led2 Alias Portb.7                      'zum Setzen der Ports
K1 Alias Portb.2
K2 Alias Portb.3

Declare Sub Swrmeter
Declare Sub Phasendetektor
Declare Sub Positioniere1
Declare Sub Positioniere2
Declare Sub Grundstellung1
Declare Sub Grundstellung2
Declare Sub Motorschritt1
Declare Sub Motorschritt2

'Zuweisungen
'-----------
Ddrb = &B11001111                       'PB0...3,6,7 Binrausgang
                                        'Rest Programmierstecker
Portb = &B00000011                      'PB0..1 setzen = Spule M2.2 stromlos

Ddrd = &B11111100                       'PD2...7 Binrausgang, Rest UART
Portd = &B11111100                      'PD2...7 setzen = Spule M2.1
                                        '                 Spule M1.2
                                        '                 Spule M1.1 stromlos
T1min = 2
T2min = 2
Tmax = 20
P1 = 1                                  'Grundstellung Phase Schrittmotor
P2 = 1
T1 = Tmax                               'Wartezeit pro Schritt in ms
T2 = Tmax
X1 = 0                                  'Istwert Position
X2 = 0


Led1 = Aus                              'LED Lichtschranke
Led2 = Aus
K1 = Aus                                'Relais
K2 = Aus

A1(1) = &B00000000                      'Bitmuster fr Schrittmotor 1, PORTD
A1(2) = &B00100000
A1(3) = &B00101000
A1(4) = &B00001000
A1(5) = &B00010100                      'Bremse, d.h. Kurzschlu
A1(6) = &B00111100                      'Abschaltung Strangstrom, offen

A2(1) = &B00000000                      'dto. Schrittmotor 2, low, PORTD
A2(2) = &B00000000
A2(3) = &B10000000
A2(4) = &B10000000
A2(5) = &B01000000
A2(6) = &B11000000

A3(1) = &B00000000                      'dto. Schrittmotor 2, high, PORTB
A3(2) = &B00000010
A3(3) = &B00000010
A3(4) = &B00000000
A3(5) = &B00000001
A3(6) = &B00000011

'Startsignal
'-----------
K2 = Ein                                'Relais einschalten
Wait 1
K2 = Aus

Config Input0 = Cr , Echo = Off

'Warten auf Befehle vom PC
'-------------------------
Do

   Input B Noecho                       'warten, bis PC-Befehl eintrifft

   If Left(b , 2) = "P1" Then           'Positionieren Motor 1
      B = Mid(b , 3)                    'Sollwert extrahieren
      W1 = Val(b)                       'in numerische Variable wandeln
      Gosub Positioniere1
      Print "P1" ; X1                   'Quittierung an PC
   End If

   If Left(b , 2) = "P2" Then           'Positionieren Motor 2
      B = Mid(b , 3)
      W2 = Val(b)
      Gosub Positioniere2
      Print "P2" ; X2
   End If

   If Left(b , 2) = "G1" Then           'Grundstellung Motor 1
      Gosub Grundstellung1
      Print "G1" ; X1                   'errechneter Wert
      X1 = 0                            'wahrer Wert
   End If

   If Left(b , 2) = "G2" Then           'Grundstellung Motor 2
      Gosub Grundstellung2
      Print "G2" ; X2                   'errechneter Wert
      X2 = 0                            'wahrer Wert
   End If

   If Left(b , 2) = "K1" Then           'Relais K1 (Akku)
      B = Mid(b , 3)
      H = Val(b)
      K1 = H
      Print "K1" ; H
   End If

    If Left(b , 2) = "K2" Then          'Relais K2
      B = Mid(b , 3)
      H = Val(b)
      K2 = H
      Print "K2" ; H
   End If

   If Left(b , 3) = "SWR" Then          'SWR
      Gosub Swrmeter
      Print "SWR" ; C ; "/" ; D
   End If

   If Left(b , 2) = "PD" Then           'Phase
      Gosub Phasendetektor
      Print "PD" ; E ; "/" ; F
   End If
Loop

End

'SWR-Werte auslesen
'------------------
Sub Swrmeter
   C = Getadc(0)
   D = Getadc(1)
End Sub

'Werte Phasendetektor auslesen
'-----------------------------
Sub Phasendetektor
   E = Getadc(2)
   F = Getadc(3)
End Sub

'Positionierung Schrittmotor 1
'-----------------------------
Sub Positioniere1
   P1 = M1                              'letzte aktive Phase

   'Rechtslauf
   '----------
   While W1 > X1
      Incr P1                           'Phasenzhler
      If P1 > 4 Then P1 = 1             'Umlauf

      'Geschwindigkeitsanpassung
      '-------------------------
      Delta = W1 - X1                   'Differenz zum Ziel
      If Delta < Tmax Then
         Incr T1                        'langsamer werden
      Else
         If T1 > T1min Then Decr T1     'schneller werden
      End If

      'Ansteuerung Motor1
      '------------------
      Motorschritt1
      Incr X1
      Waitms T1
   Wend

   'Linkslauf
   '---------
   While W1 < X1
      Decr P1
      If P1 < 1 Then P1 = 4
      Delta = X1 - W1
      If Delta < Tmax Then
         Incr T1
      Else
         If T1 > T1min Then Decr T1
      End If
      Motorschritt1
      Decr X1
      Waitms T1
   Wend

   M1 = P1
   P1 = 5                               'Bremse
   T1 = Tmax                            'langsamste Geschwindigkeit
   Motorschritt1
End Sub

'Positionierung Schrittmotor 2
'-----------------------------
Sub Positioniere2
   P2 = M2
   While W2 > X2                        'Rechtslauf
      Incr P2
      If P2 > 4 Then P2 = 1

      Delta = W2 - X2                   'Differenz zum Ziel
      If Delta < Tmax Then
         Incr T2                        'langsamer werden
      Else
         If T2 > T2min Then Decr T2     'schneller werden
      End If

      Motorschritt2
      Incr X2
      Waitms T2
   Wend
   While W2 < X2                        'Linkslauf
      Decr P2
      If P2 < 1 Then P2 = 4

      Delta = X2 - W2
      If Delta < Tmax Then
         Incr T2
      Else
         If T2 > T2min Then Decr T2
      End If

      Motorschritt2
      Decr X2
      Waitms T2
   Wend
   M2 = P2                              'aktive Phase merken
   P2 = 5                               'Bremse
   T2 = Tmax
   Motorschritt2
End Sub

'Grundstellung Motor 1 suchen
'----------------------------
Sub Grundstellung1
   Led1 = Ein                           'LEDs Lichtschranken einschalten
   Waitms 300

   If Getadc(5) > 150 Then              'Lichtschranke teilweise verdeckt
      W1 = 250                          'kurzer Rechtslauf
      Positioniere1
      Wait 1
   End If

   Tg = T1min * 2                       'halbe Geschwindigkeit
   Do
      If Getadc(5) > 250 Then Exit Do   'Linkslauf, bis Lichtschranke anspricht
      Decr X1                           'mitzhlen
      Decr P1
      If P1 < 1 Then P1 = 4
      If T1 > Tg Then Decr T1           'auf halbe Geschwindigkeit begrenzen
      Motorschritt1                     'verfahren
      Waitms T1
   Loop

   Led1 = Aus                           'LEDs Lichtschranken ausschalten
   M1 = P1                              'aktive Phase merken
   P1 = 5                               'Bremse
   T1 = Tmax
   Motorschritt1                        'Kommando ausfhren
End Sub

'Grundstellung Motor 2 suchen
'----------------------------
Sub Grundstellung2
   Led2 = Ein                           'LEDs Lichtschranken einschalten
   Waitms 300

   If Getadc(4) > 150 Then              'Lichtschranke teilweise verdeckt
      W2 = 20                           'kurzer Rechtslauf
      Positioniere2
      Wait 1
   End If

   Tg = T2min * 3                       '1/3 Geschwindigkeit
   Do
      If Getadc(4) > 250 Then Exit Do   'Linkslauf, bis Lichtschranke anspricht
      Decr X2                           'mitzhlen
      Decr P2
      If P2 < 1 Then P2 = 4
      If T2 > Tg Then Decr T2           'Geschwindigkeit begrenzen
      Motorschritt2                     'verfahren
      Waitms T2
   Loop                                 'Grundstellung mit Tmin erreicht

   Led2 = Aus                           'LEDs Lichtschranken ausschalten
   M2 = P2                              'aktive Phase merken
   P2 = 5                               'Bremse
   T2 = Tmax
   Motorschritt2                        'Kommando ausfhren
End Sub

'setzen Motorschritt1
'--------------------
Sub Motorschritt1
   Temp1 = Portd And &B11000000         'bit 2,3,4,5 fr M1 in PORTD nullen
   Portd = Temp1 + A1(p1)               'kombinieren
End Sub

'setzen Motorschritt2
'--------------------
Sub Motorschritt2
   Temp1 = Portd And &B00111111         'bit 6 und 7 fr M2 in PORTD nullen
   Temp2 = Temp1 + A2(p2)               'kombinieren

   Temp3 = Portb And &B11111100         'bit 0 und 1 fr M2 in PORTB nullen
   Temp4 = Temp3 + A3(p2)               'kombinieren

   Portd = Temp2                        'zuweisen
   Portb = Temp4
End Sub